@@ -3,22 +3,23 @@ from functools import partial
 from typing import TYPE_CHECKING
 from typing import Any
 from typing import Dict
+from typing import Iterable
 from typing import List
 from typing import Optional
+from typing import cast
 
 from isodate.isodatetime import parse_datetime
+from jsonschema._types import is_array
+from jsonschema._types import is_bool
+from jsonschema._types import is_integer
+from jsonschema._types import is_number
+from jsonschema._types import is_object
 from jsonschema.protocols import Validator
 from openapi_schema_validator._format import oas30_format_checker
-from openapi_schema_validator._types import is_array
-from openapi_schema_validator._types import is_bool
-from openapi_schema_validator._types import is_integer
-from openapi_schema_validator._types import is_number
-from openapi_schema_validator._types import is_object
 from openapi_schema_validator._types import is_string
 
-from openapi_core.extensions.models.factories import ModelFactory
+from openapi_core.extensions.models.factories import ModelClassImporter
 from openapi_core.schema.schemas import get_all_properties
-from openapi_core.schema.schemas import get_all_properties_names
 from openapi_core.spec import Spec
 from openapi_core.unmarshalling.schemas.datatypes import FormattersDict
 from openapi_core.unmarshalling.schemas.enums import UnmarshalContext
@@ -196,10 +197,19 @@ class ObjectUnmarshaller(ComplexUnmarshaller):
     }
 
     @property
-    def model_factory(self) -> ModelFactory:
-        return ModelFactory()
+    def object_class_factory(self) -> ModelClassImporter:
+        return ModelClassImporter()
 
     def unmarshal(self, value: Any) -> Any:
+        properties = self.unmarshal_raw(value)
+
+        model = self.schema.getkey("x-model")
+        fields: Iterable[str] = properties and properties.keys() or []
+        object_class = self.object_class_factory.create(fields, model=model)
+
+        return object_class(**properties)
+
+    def unmarshal_raw(self, value: Any) -> Any:
         try:
             value = self.formatter.unmarshal(value)
         except ValueError as exc:
@@ -208,64 +218,54 @@ class ObjectUnmarshaller(ComplexUnmarshaller):
         else:
             return self._unmarshal_object(value)
 
+    def _clone(self, schema: Spec) -> "ObjectUnmarshaller":
+        return cast(
+            "ObjectUnmarshaller",
+            self.unmarshallers_factory.create(schema, "object"),
+        )
+
     def _unmarshal_object(self, value: Any) -> Any:
+        properties = {}
+
         if "oneOf" in self.schema:
-            properties = None
+            one_of_properties = None
             for one_of_schema in self.schema / "oneOf":
                 try:
-                    unmarshalled = self._unmarshal_properties(
-                        value, one_of_schema
+                    unmarshalled = self._clone(one_of_schema).unmarshal_raw(
+                        value
                     )
                 except (UnmarshalError, ValueError):
                     pass
                 else:
-                    if properties is not None:
+                    if one_of_properties is not None:
                         log.warning("multiple valid oneOf schemas found")
                         continue
-                    properties = unmarshalled
+                    one_of_properties = unmarshalled
 
-            if properties is None:
+            if one_of_properties is None:
                 log.warning("valid oneOf schema not found")
+            else:
+                properties.update(one_of_properties)
 
-        else:
-            properties = self._unmarshal_properties(value)
+        elif "anyOf" in self.schema:
+            any_of_properties = None
+            for any_of_schema in self.schema / "anyOf":
+                try:
+                    unmarshalled = self._clone(any_of_schema).unmarshal_raw(
+                        value
+                    )
+                except (UnmarshalError, ValueError):
+                    pass
+                else:
+                    any_of_properties = unmarshalled
+                    break
 
-        if "x-model" in self.schema:
-            name = self.schema["x-model"]
-            return self.model_factory.create(properties, name=name)
+            if any_of_properties is None:
+                log.warning("valid anyOf schema not found")
+            else:
+                properties.update(any_of_properties)
 
-        return properties
-
-    def _unmarshal_properties(
-        self, value: Any, one_of_schema: Optional[Spec] = None
-    ) -> Dict[str, Any]:
-        all_props = get_all_properties(self.schema)
-        all_props_names = get_all_properties_names(self.schema)
-
-        if one_of_schema is not None:
-            all_props.update(get_all_properties(one_of_schema))
-            all_props_names |= get_all_properties_names(one_of_schema)
-
-        value_props_names = list(value.keys())
-        extra_props = set(value_props_names) - set(all_props_names)
-
-        properties: Dict[str, Any] = {}
-        additional_properties = self.schema.getkey(
-            "additionalProperties", True
-        )
-        if isinstance(additional_properties, dict):
-            additional_prop_schema = self.schema / "additionalProperties"
-            for prop_name in extra_props:
-                prop_value = value[prop_name]
-                properties[prop_name] = self.unmarshallers_factory.create(
-                    additional_prop_schema
-                )(prop_value)
-        elif additional_properties is True:
-            for prop_name in extra_props:
-                prop_value = value[prop_name]
-                properties[prop_name] = prop_value
-
-        for prop_name, prop in list(all_props.items()):
+        for prop_name, prop in get_all_properties(self.schema).items():
             read_only = prop.getkey("readOnly", False)
             if self.context == UnmarshalContext.REQUEST and read_only:
                 continue
@@ -283,6 +283,24 @@ class ObjectUnmarshaller(ComplexUnmarshaller):
                 prop_value
             )
 
+        additional_properties = self.schema.getkey(
+            "additionalProperties", True
+        )
+        if additional_properties is not False:
+            # free-form object
+            if additional_properties is True:
+                additional_prop_schema = Spec.from_dict({})
+            # defined schema
+            else:
+                additional_prop_schema = self.schema / "additionalProperties"
+            additional_prop_unmarshaler = self.unmarshallers_factory.create(
+                additional_prop_schema
+            )
+            for prop_name, prop_value in value.items():
+                if prop_name in properties:
+                    continue
+                properties[prop_name] = additional_prop_unmarshaler(prop_value)
+
         return properties
 
 
@@ -302,6 +320,10 @@ class AnyUnmarshaller(ComplexUnmarshaller):
         if one_of_schema:
             return self.unmarshallers_factory.create(one_of_schema)(value)
 
+        any_of_schema = self._get_any_of_schema(value)
+        if any_of_schema:
+            return self.unmarshallers_factory.create(any_of_schema)(value)
+
         all_of_schema = self._get_all_of_schema(value)
         if all_of_schema:
             return self.unmarshallers_factory.create(all_of_schema)(value)
@@ -336,6 +358,21 @@ class AnyUnmarshaller(ComplexUnmarshaller):
                 return subschema
         return None
 
+    def _get_any_of_schema(self, value: Any) -> Optional[Spec]:
+        if "anyOf" not in self.schema:
+            return None
+
+        any_of_schemas = self.schema / "anyOf"
+        for subschema in any_of_schemas:
+            unmarshaller = self.unmarshallers_factory.create(subschema)
+            try:
+                unmarshaller.validate(value)
+            except ValidateError:
+                continue
+            else:
+                return subschema
+        return None
+
     def _get_all_of_schema(self, value: Any) -> Optional[Spec]:
         if "allOf" not in self.schema:
             return None
